@@ -45,8 +45,8 @@ namespace functionspace {
 namespace detail {
 
 namespace {
-template <typename T>
-array::LocalView<T, 3> make_leveled_view( const Field& field ) {
+template <typename T, typename Field>
+array::LocalView<T, 3> make_leveled_view( Field& field ) {
     using namespace array;
     if ( field.levels() ) {
         if ( field.variables() ) {
@@ -79,6 +79,7 @@ public:
         return inst;
     }
     util::ObjectHandle<value_type> get_or_create( const Mesh& mesh ) {
+        registerMesh( *mesh.get() );
         creator_type creator = std::bind( &CellColumnsHaloExchangeCache::create, mesh );
         return Base::get_or_create( key( *mesh.get() ), creator );
     }
@@ -92,7 +93,6 @@ private:
     }
 
     static value_type* create( const Mesh& mesh ) {
-        mesh.get()->attachObserver( instance() );
         value_type* value = new value_type();
         value->setup( array::make_view<int, 1>( mesh.cells().partition() ).data(),
                       array::make_view<idx_t, 1>( mesh.cells().remote_index() ).data(), REMOTE_IDX_BASE,
@@ -113,6 +113,7 @@ public:
         return inst;
     }
     util::ObjectHandle<value_type> get_or_create( const Mesh& mesh ) {
+        registerMesh( *mesh.get() );
         creator_type creator = std::bind( &CellColumnsGatherScatterCache::create, mesh );
         return Base::get_or_create( key( *mesh.get() ), creator );
     }
@@ -126,7 +127,6 @@ private:
     }
 
     static value_type* create( const Mesh& mesh ) {
-        mesh.get()->attachObserver( instance() );
         value_type* value = new value_type();
         value->setup( array::make_view<int, 1>( mesh.cells().partition() ).data(),
                       array::make_view<idx_t, 1>( mesh.cells().remote_index() ).data(), REMOTE_IDX_BASE,
@@ -147,6 +147,7 @@ public:
         return inst;
     }
     util::ObjectHandle<value_type> get_or_create( const Mesh& mesh ) {
+        registerMesh( *mesh.get() );
         creator_type creator = std::bind( &CellColumnsChecksumCache::create, mesh );
         return Base::get_or_create( key( *mesh.get() ), creator );
     }
@@ -160,7 +161,6 @@ private:
     }
 
     static value_type* create( const Mesh& mesh ) {
-        mesh.get()->attachObserver( instance() );
         value_type* value = new value_type();
         util::ObjectHandle<parallel::GatherScatter> gather(
             CellColumnsGatherScatterCache::instance().get_or_create( mesh ) );
@@ -192,7 +192,7 @@ void CellColumns::set_field_metadata( const eckit::Configuration& config, Field&
 }
 
 idx_t CellColumns::config_size( const eckit::Configuration& config ) const {
-    const idx_t rank = static_cast<idx_t>( mpi::comm().rank() );
+    const idx_t rank = mpi::rank();
     idx_t size       = nb_cells();
     bool global( false );
     if ( config.get( "global", global ) ) {
@@ -247,10 +247,7 @@ array::ArrayShape CellColumns::config_shape( const eckit::Configuration& config
 }
 
 CellColumns::CellColumns( const Mesh& mesh, const eckit::Configuration& config ) :
-    mesh_( mesh ),
-    cells_( mesh_.cells() ),
-    nb_levels_( config.getInt( "levels", 0 ) ),
-    nb_cells_( 0 ) {
+    mesh_( mesh ), cells_( mesh_.cells() ), nb_levels_( config.getInt( "levels", 0 ) ), nb_cells_( 0 ) {
     ATLAS_TRACE();
     if ( config.has( "halo" ) ) {
         halo_ = mesh::Halo( config.getInt( "halo" ) );
@@ -389,22 +386,22 @@ void CellColumns::gather( const FieldSet& local_fieldset, FieldSet& global_field
         idx_t root( 0 );
         glb.metadata().get( "owner", root );
         if ( loc.datatype() == array::DataType::kind<int>() ) {
-            parallel::Field<int const> loc_field( make_leveled_view<int>( loc ) );
+            parallel::Field<int const> loc_field( make_leveled_view<const int>( loc ) );
             parallel::Field<int> glb_field( make_leveled_view<int>( glb ) );
             gather().gather( &loc_field, &glb_field, nb_fields, root );
         }
         else if ( loc.datatype() == array::DataType::kind<long>() ) {
-            parallel::Field<long const> loc_field( make_leveled_view<long>( loc ) );
+            parallel::Field<long const> loc_field( make_leveled_view<const long>( loc ) );
             parallel::Field<long> glb_field( make_leveled_view<long>( glb ) );
             gather().gather( &loc_field, &glb_field, nb_fields, root );
         }
         else if ( loc.datatype() == array::DataType::kind<float>() ) {
-            parallel::Field<float const> loc_field( make_leveled_view<float>( loc ) );
+            parallel::Field<float const> loc_field( make_leveled_view<const float>( loc ) );
             parallel::Field<float> glb_field( make_leveled_view<float>( glb ) );
             gather().gather( &loc_field, &glb_field, nb_fields, root );
         }
         else if ( loc.datatype() == array::DataType::kind<double>() ) {
-            parallel::Field<double const> loc_field( make_leveled_view<double>( loc ) );
+            parallel::Field<double const> loc_field( make_leveled_view<const double>( loc ) );
             parallel::Field<double> glb_field( make_leveled_view<double>( glb ) );
             gather().gather( &loc_field, &glb_field, nb_fields, root );
         }
@@ -447,22 +444,22 @@ void CellColumns::scatter( const FieldSet& global_fieldset, FieldSet& local_fiel
         glb.metadata().get( "owner", root );
 
         if ( loc.datatype() == array::DataType::kind<int>() ) {
-            parallel::Field<int const> glb_field( make_leveled_view<int>( glb ) );
+            parallel::Field<int const> glb_field( make_leveled_view<const int>( glb ) );
             parallel::Field<int> loc_field( make_leveled_view<int>( loc ) );
             scatter().scatter( &glb_field, &loc_field, nb_fields, root );
         }
         else if ( loc.datatype() == array::DataType::kind<long>() ) {
-            parallel::Field<long const> glb_field( make_leveled_view<long>( glb ) );
+            parallel::Field<long const> glb_field( make_leveled_view<const long>( glb ) );
             parallel::Field<long> loc_field( make_leveled_view<long>( loc ) );
             scatter().scatter( &glb_field, &loc_field, nb_fields, root );
         }
         else if ( loc.datatype() == array::DataType::kind<float>() ) {
-            parallel::Field<float const> glb_field( make_leveled_view<float>( glb ) );
+            parallel::Field<float const> glb_field( make_leveled_view<const float>( glb ) );
             parallel::Field<float> loc_field( make_leveled_view<float>( loc ) );
             scatter().scatter( &glb_field, &loc_field, nb_fields, root );
         }
         else if ( loc.datatype() == array::DataType::kind<double>() ) {
-            parallel::Field<double const> glb_field( make_leveled_view<double>( glb ) );
+            parallel::Field<double const> glb_field( make_leveled_view<const double>( glb ) );
             parallel::Field<double> loc_field( make_leveled_view<double>( loc ) );
             scatter().scatter( &glb_field, &loc_field, nb_fields, root );
         }
@@ -485,9 +482,9 @@ void CellColumns::scatter( const Field& global, Field& local ) const {
 namespace {
 template <typename T>
 std::string checksum_3d_field( const parallel::Checksum& checksum, const Field& field ) {
-    array::ArrayView<T, 3> values = array::make_view<T, 3>( field );
+    auto values = array::make_view<const T, 3>( field );
     array::ArrayT<T> surface_field( field.shape( 0 ), field.shape( 2 ) );
-    array::ArrayView<T, 2> surface = array::make_view<T, 2>( surface_field );
+    auto surface = array::make_view<T, 2>( surface_field );
     for ( idx_t n = 0; n < values.shape( 0 ); ++n ) {
         for ( idx_t j = 0; j < surface.shape( 1 ); ++j ) {
             surface( n, j ) = 0.;
@@ -500,7 +497,7 @@ std::string checksum_3d_field( const parallel::Checksum& checksum, const Field&
 }
 template <typename T>
 std::string checksum_2d_field( const parallel::Checksum& checksum, const Field& field ) {
-    array::ArrayView<T, 2> values = array::make_view<T, 2>( field );
+    auto values = array::make_view<T, 2>( field );
     return checksum.execute( values.data(), field.stride( 0 ) );
 }
 
@@ -562,6 +559,10 @@ const parallel::Checksum& CellColumns::checksum() const {
     return *checksum_;
 }
 
+Field CellColumns::lonlat() const {
+    return mesh_.cells().field( "lonlat" );
+}
+
 //------------------------------------------------------------------------------
 //------------------------------------------------------------------------------
 //------------------------------------------------------------------------------
@@ -769,8 +770,7 @@ void atlas__fs__CellColumns__checksum_field( const CellColumns* This, const fiel
 CellColumns::CellColumns() : FunctionSpace(), functionspace_( nullptr ) {}
 
 CellColumns::CellColumns( const FunctionSpace& functionspace ) :
-    FunctionSpace( functionspace ),
-    functionspace_( dynamic_cast<const detail::CellColumns*>( get() ) ) {}
+    FunctionSpace( functionspace ), functionspace_( dynamic_cast<const detail::CellColumns*>( get() ) ) {}
 
 CellColumns::CellColumns( const Mesh& mesh, const eckit::Configuration& config ) :
     FunctionSpace( new detail::CellColumns( mesh, config ) ),
