@@ -7,9 +7,9 @@
  * granted to it by virtue of its status as an intergovernmental organisation
  * nor does it submit to any jurisdiction.
  */
-
 #include <algorithm>
 #include <cmath>
+#include <iomanip>
 #include <limits>
 #include <memory>
 #include <numeric>
@@ -77,6 +77,11 @@ StructuredMeshGenerator::StructuredMeshGenerator( const eckit::Parametrisation&
         options.set( "patch_pole", patch_pole );
     }
 
+    bool patch_quads;
+    if ( p.get( "patch_quads", patch_quads ) ) {
+        options.set( "patch_quads", patch_quads );
+    }
+
     bool unique_pole;
     if ( p.get( "unique_pole", unique_pole ) ) {
         options.set( "unique_pole", unique_pole );
@@ -160,23 +165,22 @@ void StructuredMeshGenerator::configure_defaults() {
     options.set( "3d", false );
 
     // This option sets number of parts the mesh will be split in
-    options.set( "nb_parts", mpi::comm().size() );
+    options.set( "nb_parts", mpi::size() );
 
     // This option sets the part that will be generated
-    options.set( "part", mpi::comm().rank() );
+    options.set( "part", mpi::rank() );
 
-    // Experimental option. The result is a non-standard Reduced Gaussian Grid,
-    // with a ragged Greenwich line
-    options.set( "stagger", false );
+    // Experimental option. This is what makes tripolar grid patched with quads
+    options.set( "patch_quads", false );
 
     // This option sets the maximum angle deviation for a quadrilateral element
     // angle = 30  -->  minimises number of triangles
     // angle = 0   -->  maximises number of triangles
-    options.set<double>( "angle", 0. );
+    options.set( "angle", 0. );
 
-    options.set<bool>( "triangulate", false );
+    options.set( "triangulate", false );
 
-    options.set<bool>( "ghost_at_end", true );
+    options.set( "ghost_at_end", true );
 }
 
 void StructuredMeshGenerator::generate( const Grid& grid, Mesh& mesh ) const {
@@ -184,19 +188,24 @@ void StructuredMeshGenerator::generate( const Grid& grid, Mesh& mesh ) const {
 
     const StructuredGrid rg = StructuredGrid( grid );
     if ( !rg ) {
-        throw_Exception( "Structured can only work with a Structured", Here() );
+        throw_Exception( "Structured can only work with a Structured grid", Here() );
     }
 
-    idx_t nb_parts = options.get<idx_t>( "nb_parts" );
+    idx_t nb_parts = options.getInt( "nb_parts" );
 
     std::string partitioner_type = "equal_regions";
     options.get( "partitioner", partitioner_type );
 
-    if ( partitioner_type == "trans" && rg.ny() % 2 == 1 ) {
-        partitioner_type = "equal_regions";  // Odd number of latitudes
+    if ( partitioner_type == "trans" ) {
+        if ( rg.ny() % 2 == 1 ) {
+            partitioner_type = "equal_regions";  // Odd number of latitudes
+        }
+        if ( nb_parts != mpi::size() ) {
+            partitioner_type = "equal_regions";
+        }
     }
-    if ( nb_parts == 1 || mpi::comm().size() == 1 ) {
-        partitioner_type = "equal_regions";  // Only one part --> Trans is slower
+    if ( nb_parts == 1 ) {
+        partitioner_type = "serial";
     }
 
     grid::Partitioner partitioner( partitioner_type, nb_parts );
@@ -219,25 +228,25 @@ void StructuredMeshGenerator::generate( const Grid& grid, const grid::Distributi
 
     ATLAS_ASSERT( !mesh.generated() );
 
-    if ( grid.size() != idx_t( distribution.partition().size() ) ) {
+    if ( grid.size() != idx_t( distribution.size() ) ) {
         std::stringstream msg;
         msg << "Number of points in grid (" << grid.size()
             << ") different from "
                "number of points in grid distribution ("
-            << distribution.partition().size() << ")";
+            << distribution.size() << ")";
         throw_AssertionFailed( msg.str(), Here() );
     }
 
-    idx_t mypart = options.get<idx_t>( "part" );
+    idx_t mypart = options.getInt( "part" );
 
 // show distribution
 #if DEBUG_OUTPUT
-    int inode              = 0;
-    std::vector<int> parts = distribution;
+    int inode                       = 0;
+    const atlas::vector<int>& parts = distribution;
     Log::info() << "Partition : " << std::endl;
     for ( size_t ilat = 0; ilat < rg.ny(); ilat++ ) {
         for ( size_t ilon = 0; ilon < rg.nx( ilat ); ilon++ ) {
-            Log::info() << std::setw( 3 ) << parts[inode];
+            Log::info() << std::setw( 3 ) << distribution.partition( inode );
             inode++;
         }
         Log::info() << std::endl;
@@ -253,16 +262,16 @@ void StructuredMeshGenerator::generate( const Grid& grid, const grid::Distributi
     generate_mesh( rg, distribution, region, mesh );
 }
 
-void StructuredMeshGenerator::generate_region( const StructuredGrid& rg, const std::vector<int>& parts, int mypart,
-                                               Region& region ) const {
+void StructuredMeshGenerator::generate_region( const StructuredGrid& rg, const grid::Distribution& distribution,
+                                               int mypart, Region& region ) const {
     ATLAS_TRACE();
 
-    double max_angle       = options.get<double>( "angle" );
-    bool triangulate_quads = options.get<bool>( "triangulate" );
-    bool three_dimensional = options.get<bool>( "3d" );
-    bool has_north_pole    = eckit::types::is_approximately_equal( rg.y().front(), 90. );
-    bool has_south_pole    = eckit::types::is_approximately_equal( rg.y().back(), -90. );
-    bool unique_pole = options.get<bool>( "unique_pole" ) && three_dimensional && has_north_pole && has_south_pole;
+    double max_angle        = options.getDouble( "angle" );
+    bool triangulate_quads  = options.getBool( "triangulate" );
+    bool three_dimensional  = options.getBool( "3d" );
+    bool has_north_pole     = eckit::types::is_approximately_equal( rg.y().front(), 90. );
+    bool has_south_pole     = eckit::types::is_approximately_equal( rg.y().back(), -90. );
+    bool unique_pole        = options.getBool( "unique_pole" ) && three_dimensional && has_north_pole && has_south_pole;
     bool periodic_east_west = rg.periodic();
 
     int n;
@@ -273,7 +282,7 @@ Find min and max latitudes used by this part.
     idx_t lat_north = -1;
     for ( idx_t jlat = 0; jlat < rg.ny(); ++jlat ) {
         for ( idx_t jlon = 0; jlon < rg.nx( jlat ); ++jlon ) {
-            if ( parts.at( n ) == mypart ) {
+            if ( distribution.partition( n ) == mypart ) {
                 lat_north = jlat;
                 goto end_north;
             }
@@ -286,7 +295,7 @@ end_north:
     idx_t lat_south = -1;
     for ( idx_t jlat = rg.ny() - 1; jlat >= 0; --jlat ) {
         for ( idx_t jlon = rg.nx( jlat ) - 1; jlon >= 0; --jlon ) {
-            if ( parts.at( n ) == mypart ) {
+            if ( distribution.partition( n ) == mypart ) {
                 lat_south = jlat;
                 goto end_south;
             }
@@ -301,7 +310,7 @@ end_south:
     for ( idx_t jlat = 0; jlat < rg.ny(); ++jlat ) {
         offset.at( jlat ) = n;
         n += rg.nx( jlat );
-    };
+    }
 
     /*
 We need to connect to next region
@@ -329,7 +338,6 @@ We need to connect to next region
     array::ArrayView<int, 3> elemview = array::make_view<int, 3>( *region.elems );
     elemview.assign( -1 );
 
-    bool stagger = options.get<bool>( "stagger" );
     for ( idx_t jlat = lat_north; jlat < lat_south; ++jlat ) {
         //    std::stringstream filename; filename << "/tmp/debug/"<<jlat;
 
@@ -369,7 +377,7 @@ We need to connect to next region
         ipS2 = std::min( ipS1 + 1, endS );
 
         idx_t jelem = 0;
-        int pE      = parts.at( offset.at( latN ) );
+        int pE      = distribution.partition( offset.at( latN ) );
 
 #if DEBUG_OUTPUT
         Log::info() << "=================\n";
@@ -390,29 +398,29 @@ We need to connect to next region
 
             int pN1, pS1, pN2, pS2;
             if ( ipN1 != rg.nx( latN ) ) {
-                pN1 = parts.at( offset.at( latN ) + ipN1 );
+                pN1 = distribution.partition( offset.at( latN ) + ipN1 );
             }
             else {
-                pN1 = parts.at( offset.at( latN ) );
+                pN1 = distribution.partition( offset.at( latN ) );
             }
             if ( ipS1 != rg.nx( latS ) ) {
-                pS1 = parts.at( offset.at( latS ) + ipS1 );
+                pS1 = distribution.partition( offset.at( latS ) + ipS1 );
             }
             else {
-                pS1 = parts.at( offset.at( latS ) );
+                pS1 = distribution.partition( offset.at( latS ) );
             }
 
             if ( ipN2 == rg.nx( latN ) ) {
-                pN2 = parts.at( offset.at( latN ) );
+                pN2 = distribution.partition( offset.at( latN ) );
             }
             else {
-                pN2 = parts.at( offset.at( latN ) + ipN2 );
+                pN2 = distribution.partition( offset.at( latN ) + ipN2 );
             }
             if ( ipS2 == rg.nx( latS ) ) {
-                pS2 = parts.at( offset.at( latS ) );
+                pS2 = distribution.partition( offset.at( latS ) );
             }
             else {
-                pS2 = parts.at( offset.at( latS ) + ipS2 );
+                pS2 = distribution.partition( offset.at( latS ) + ipS2 );
             }
 
             // Log::info()  << ipN1 << "("<<pN1<<") " << ipN2 <<"("<<pN2<<")" <<  std::endl;
@@ -428,19 +436,6 @@ We need to connect to next region
             xS1 = rg.x( ipS1, latS ) * to_rad;
             xS2 = rg.x( ipS2, latS ) * to_rad;
 
-            if ( stagger && ( latN + 1 ) % 2 == 0 ) {
-                xN1 += M_PI / static_cast<double>( rg.nx( latN ) );
-            }
-            if ( stagger && ( latN + 1 ) % 2 == 0 ) {
-                xN2 += M_PI / static_cast<double>( rg.nx( latN ) );
-            }
-            if ( stagger && ( latS + 1 ) % 2 == 0 ) {
-                xS1 += M_PI / static_cast<double>( rg.nx( latS ) );
-            }
-            if ( stagger && ( latS + 1 ) % 2 == 0 ) {
-                xS2 += M_PI / static_cast<double>( rg.nx( latS ) );
-            }
-
 #if DEBUG_OUTPUT
             Log::info() << "-------\n";
 #endif
@@ -529,7 +524,20 @@ We need to connect to next region
                     try_make_triangle_down = true;
                 }
                 else {
-                    throw_Exception( "Should not try to make a quadrilateral!", Here() );
+                    if ( ipN1 == ipN2 ) {
+                        try_make_triangle_up = true;
+                    }
+                    else if ( ipS1 == ipS2 ) {
+                        try_make_triangle_down = true;
+                    }
+                    else {
+                        ATLAS_DEBUG_VAR( dN1S2 );
+                        ATLAS_DEBUG_VAR( dS1N2 );
+                        ATLAS_DEBUG_VAR( jlat );
+                        Log::info() << ipN1 << "(" << xN1 << ")  " << ipN2 << "(" << xN2 << ")  " << std::endl;
+                        Log::info() << ipS1 << "(" << xS1 << ")  " << ipS2 << "(" << xS2 << ")  " << std::endl;
+                        throw_Exception( "Should not try to make a quadrilateral!", Here() );
+                    }
                 }
             }
             // ------------------------------------------------
@@ -556,7 +564,7 @@ We need to connect to next region
                 std::array<int, 4> np{pN1, pN2, pS1, pS2};
                 std::array<int, 4> pcnts;
                 for ( int j = 0; j < 4; ++j ) {
-                    pcnts[j] = std::count( np.begin(), np.end(), np[j] );
+                    pcnts[j] = static_cast<int>( std::count( np.begin(), np.end(), np[j] ) );
                 }
                 if ( pcnts[0] > 2 ) {  // 3 or more of pN1
                     pE = pN1;
@@ -743,9 +751,9 @@ We need to connect to next region
     int nb_region_nodes = 0;
     for ( int jlat = region.north; jlat <= region.south; ++jlat ) {
         n                           = offset.at( jlat );
-        region.lat_begin.at( jlat ) = std::max( 0, region.lat_begin.at( jlat ) );
+        region.lat_begin.at( jlat ) = std::max<idx_t>( 0, region.lat_begin.at( jlat ) );
         for ( idx_t jlon = 0; jlon < rg.nx( jlat ); ++jlon ) {
-            if ( parts.at( n ) == mypart ) {
+            if ( distribution.partition( n ) == mypart ) {
                 region.lat_begin.at( jlat ) = std::min( region.lat_begin.at( jlat ), jlon );
                 region.lat_end.at( jlat )   = std::max( region.lat_end.at( jlat ), jlon );
             }
@@ -783,37 +791,54 @@ struct GhostNode {
 };
 }  // namespace
 
-void StructuredMeshGenerator::generate_mesh( const StructuredGrid& rg, const std::vector<int>& parts,
+void StructuredMeshGenerator::generate_mesh( const StructuredGrid& rg, const grid::Distribution& distribution,
                                              const Region& region, Mesh& mesh ) const {
     ATLAS_TRACE();
 
     ATLAS_ASSERT( !mesh.generated() );
 
-    int mypart = options.get<size_t>( "part" );
-    int nparts = options.get<size_t>( "nb_parts" );
+    int mypart = options.getInt( "part" );
+    int nparts = options.getInt( "nb_parts" );
     int n, l;
+    const int y_numbering = ( rg.y().front() < rg.y().back() ) ? +1 : -1;
+    double y_north        = y_numbering < 0 ? rg.y().front() : rg.y().back();
+    double y_south        = y_numbering < 0 ? rg.y().back() : rg.y().front();
+    idx_t nx_north        = y_numbering < 0 ? rg.nx().front() : rg.nx().back();
+    idx_t nx_south        = y_numbering < 0 ? rg.nx().back() : rg.nx().front();
 
-    bool three_dimensional             = options.get<bool>( "3d" );
+    std::vector<int> part_north( nx_north );
+    std::vector<int> part_south( nx_south );
+    if ( y_numbering < 0 ) {
+        distribution.partition( 0, nx_north, part_north );
+        distribution.partition( rg.size() - nx_south, rg.size(), part_south );
+    }
+    else {
+        distribution.partition( 0, nx_south, part_south );
+        distribution.partition( rg.size() - nx_north, rg.size(), part_north );
+    }
+    bool mypart_at_north = std::any_of( part_north.begin(), part_north.end(), [&]( int p ) { return p == mypart; } );
+    bool mypart_at_south = std::any_of( part_south.begin(), part_south.end(), [&]( int p ) { return p == mypart; } );
+
+    bool three_dimensional             = options.getBool( "3d" );
     bool periodic_east_west            = rg.periodic();
     bool include_periodic_ghost_points = periodic_east_west && !three_dimensional;
     bool remove_periodic_ghost_points  = periodic_east_west && three_dimensional;
 
-    bool has_point_at_north_pole = rg.y().front() == 90 && rg.nx().front() > 0;
-    bool has_point_at_south_pole = rg.y().back() == -90 && rg.nx().back() > 0;
-    bool possible_north_pole     = !has_point_at_north_pole && rg.domain().containsNorthPole() && ( mypart == 0 );
-    bool possible_south_pole = !has_point_at_south_pole && rg.domain().containsSouthPole() && ( mypart == nparts - 1 );
+    bool has_point_at_north_pole = y_north == 90 && nx_north > 0;
+    bool has_point_at_south_pole = y_south == -90 && nx_south > 0;
+    bool possible_north_pole     = !has_point_at_north_pole && rg.domain().containsNorthPole() && mypart_at_north;
+    bool possible_south_pole     = !has_point_at_south_pole && rg.domain().containsSouthPole() && mypart_at_south;
 
     bool force_include_north_pole( options.has( "force_include_north_pole" ) &&
-                                   options.get<bool>( "force_include_north_pole" ) );
+                                   options.getBool( "force_include_north_pole" ) );
     bool force_include_south_pole( options.has( "force_include_south_pole" ) &&
-                                   options.get<bool>( "force_include_south_pole" ) );
+                                   options.getBool( "force_include_south_pole" ) );
 
-    bool include_north_pole =
-        ( possible_north_pole && options.get<bool>( "include_pole" ) ) || force_include_north_pole;
-    bool include_south_pole =
-        ( possible_south_pole && options.get<bool>( "include_pole" ) ) || force_include_south_pole;
-    bool patch_north_pole = possible_north_pole && options.get<bool>( "patch_pole" ) && rg.nx( 1 ) > 0;
-    bool patch_south_pole = possible_south_pole && options.get<bool>( "patch_pole" ) && rg.nx( rg.ny() - 2 ) > 0;
+    bool include_north_pole = ( possible_north_pole && options.getBool( "include_pole" ) ) || force_include_north_pole;
+    bool include_south_pole = ( possible_south_pole && options.getBool( "include_pole" ) ) || force_include_south_pole;
+    bool patch_north_pole   = possible_north_pole && options.getBool( "patch_pole" ) && nx_north > 0;
+    bool patch_south_pole   = possible_south_pole && options.getBool( "patch_pole" ) && nx_south > 0;
+    bool patch_quads        = options.getBool( "patch_quads" );
 
     int nnewnodes = ( !has_point_at_north_pole && include_north_pole ? 1 : 0 ) +
                     ( !has_point_at_south_pole && include_south_pole ? 1 : 0 );
@@ -827,17 +852,27 @@ void StructuredMeshGenerator::generate_mesh( const StructuredGrid& rg, const std
 
     if ( include_north_pole ) {
         ++nnodes;
-        ntriags += rg.nx( 0 ) - ( periodic_east_west ? 0 : 1 );
+        ntriags += nx_north - ( periodic_east_west ? 0 : 1 );
     }
     else if ( patch_north_pole ) {
-        ntriags += rg.nx( 0 ) - 2;
+        if ( patch_quads ) {
+            nquads += ( nx_north + 1 ) / 2;
+        }
+        else {
+            ntriags += nx_north - 2;
+        }
     }
     if ( include_south_pole ) {
         ++nnodes;
-        ntriags += rg.nx( rg.ny() - 1 ) - ( periodic_east_west ? 0 : 1 );
+        ntriags += nx_south - ( periodic_east_west ? 0 : 1 );
     }
     else if ( patch_south_pole ) {
-        ntriags += rg.nx( rg.ny() - 1 ) - 2;
+        if ( patch_quads ) {
+            nquads += ( nx_south + 1 ) / 2;
+        }
+        else {
+            ntriags += nx_south - 2;
+        }
     }
 
     size_t node_numbering_size = nnodes;
@@ -861,7 +896,7 @@ void StructuredMeshGenerator::generate_mesh( const StructuredGrid& rg, const std
     ATLAS_DEBUG_VAR( nnodes );
     ATLAS_DEBUG_VAR( ntriags );
     ATLAS_DEBUG_VAR( nquads );
-    ATLAS_DEBUG_VAR( options.get<bool>( "ghost_at_end" ) );
+    ATLAS_DEBUG_VAR( options.getBool( "ghost_at_end" ) );
 #endif
 
     std::vector<int> offset_glb( rg.ny() );
@@ -871,7 +906,7 @@ void StructuredMeshGenerator::generate_mesh( const StructuredGrid& rg, const std
     for ( idx_t jlat = 0; jlat < rg.ny(); ++jlat ) {
         offset_glb.at( jlat ) = n;
         n += rg.nx( jlat );
-    };
+    }
 
     std::vector<int> periodic_glb( rg.ny() );
 
@@ -903,10 +938,9 @@ void StructuredMeshGenerator::generate_mesh( const StructuredGrid& rg, const std
     auto flags   = array::make_view<int, 1>( nodes.flags() );
     auto halo    = array::make_view<int, 1>( nodes.halo() );
 
-    bool stagger = options.get<bool>( "stagger" );
 
     std::vector<idx_t> node_numbering( node_numbering_size, -1 );
-    if ( options.get<bool>( "ghost_at_end" ) ) {
+    if ( options.getBool( "ghost_at_end" ) ) {
         std::vector<GhostNode> ghost_nodes;
         ghost_nodes.reserve( nnodes );
         idx_t node_number = 0;
@@ -926,7 +960,7 @@ void StructuredMeshGenerator::generate_mesh( const StructuredGrid& rg, const std
             for ( idx_t jlon = region.lat_begin.at( jlat ); jlon <= region.lat_end.at( jlat ); ++jlon ) {
                 if ( jlon < rg.nx( jlat ) ) {
                     n = offset_glb.at( jlat ) + jlon;
-                    if ( parts.at( n ) == mypart ) {
+                    if ( distribution.partition( n ) == mypart ) {
                         node_numbering.at( jnode ) = node_number;
                         ++node_number;
                     }
@@ -987,9 +1021,6 @@ void StructuredMeshGenerator::generate_mesh( const StructuredGrid& rg, const std
                 double x = rg.x( jlon, jlat );
                 // std::cout << "jlat = " << jlat << "; jlon = " << jlon << "; x = " <<
                 // x << std::endl;
-                if ( stagger && ( jlat + 1 ) % 2 == 0 ) {
-                    x += 180. / static_cast<double>( rg.nx( jlat ) );
-                }
 
                 xy( inode, XX ) = x;
                 xy( inode, YY ) = y;
@@ -1001,18 +1032,29 @@ void StructuredMeshGenerator::generate_mesh( const StructuredGrid& rg, const std
                 lonlat( inode, LAT ) = crd[LAT];
 
                 glb_idx( inode ) = n + 1;
-                part( inode )    = parts.at( n );
+                part( inode )    = distribution.partition( n );
                 ghost( inode )   = 0;
                 halo( inode )    = 0;
                 Topology::reset( flags( inode ) );
-                if ( jlat == 0 && !include_north_pole ) {
-                    Topology::set( flags( inode ), Topology::BC | Topology::NORTH );
+                Topology::set( flags( inode ), Topology::BC );
+                if ( jlat == 0 ) {
+                    if ( y_numbering < 0 && !include_north_pole ) {
+                        Topology::set( flags( inode ), Topology::NORTH );
+                    }
+                    if ( y_numbering > 0 && !include_south_pole ) {
+                        Topology::set( flags( inode ), Topology::SOUTH );
+                    }
                 }
-                if ( jlat == rg.ny() - 1 && !include_south_pole ) {
-                    Topology::set( flags( inode ), Topology::BC | Topology::SOUTH );
+                if ( jlat == rg.ny() - 1 ) {
+                    if ( y_numbering < 0 && !include_south_pole ) {
+                        Topology::set( flags( inode ), Topology::SOUTH );
+                    }
+                    if ( y_numbering > 0 && !include_north_pole ) {
+                        Topology::set( flags( inode ), Topology::NORTH );
+                    }
                 }
                 if ( jlon == 0 && include_periodic_ghost_points ) {
-                    Topology::set( flags( inode ), Topology::BC | Topology::WEST );
+                    Topology::set( flags( inode ), Topology::WEST );
                 }
                 if ( part( inode ) != mypart ) {
                     Topology::set( flags( inode ), Topology::GHOST );
@@ -1025,9 +1067,6 @@ void StructuredMeshGenerator::generate_mesh( const StructuredGrid& rg, const std
                 idx_t inode = node_numbering.at( jnode );
                 // int inode_left = node_numbering.at(jnode-1);
                 double x = rg.x( rg.nx( jlat ), jlat );
-                if ( stagger && ( jlat + 1 ) % 2 == 0 ) {
-                    x += 180. / static_cast<double>( rg.nx( jlat ) );
-                }
 
                 xy( inode, XX ) = x;
                 xy( inode, YY ) = y;
@@ -1047,13 +1086,29 @@ void StructuredMeshGenerator::generate_mesh( const StructuredGrid& rg, const std
                 Topology::reset( flags( inode ) );
                 Topology::set( flags( inode ), Topology::BC | Topology::EAST );
                 Topology::set( flags( inode ), Topology::GHOST );
+                if ( jlat == 0 ) {
+                    if ( y_numbering < 0 && !include_north_pole ) {
+                        Topology::set( flags( inode ), Topology::NORTH );
+                    }
+                    if ( y_numbering > 0 && !include_south_pole ) {
+                        Topology::set( flags( inode ), Topology::SOUTH );
+                    }
+                }
+                if ( jlat == rg.ny() - 1 ) {
+                    if ( y_numbering < 0 && !include_south_pole ) {
+                        Topology::set( flags( inode ), Topology::SOUTH );
+                    }
+                    if ( y_numbering > 0 && !include_north_pole ) {
+                        Topology::set( flags( inode ), Topology::NORTH );
+                    }
+                }
                 ++jnode;
             }
             else {
                 --l;
             }
         }
-    };
+    }
 
     idx_t jnorth = -1;
     if ( include_north_pole ) {
@@ -1137,6 +1192,23 @@ void StructuredMeshGenerator::generate_mesh( const StructuredGrid& rg, const std
     idx_t quad_nodes[4];
     idx_t triag_nodes[3];
 
+    auto fix_quad_orientation = []( idx_t nodes[] ) {
+        idx_t tmp;
+        tmp      = nodes[0];
+        nodes[0] = nodes[1];
+        nodes[1] = tmp;
+        tmp      = nodes[2];
+        nodes[2] = nodes[3];
+        nodes[3] = tmp;
+    };
+    auto fix_triag_orientation = []( idx_t nodes[] ) {
+        idx_t tmp;
+        tmp      = nodes[0];
+        nodes[0] = nodes[1];
+        nodes[1] = tmp;
+    };
+
+
     for ( idx_t jlat = region.north; jlat < region.south; ++jlat ) {
         idx_t ilat  = jlat - region.north;
         idx_t jlatN = jlat;
@@ -1162,6 +1234,10 @@ void StructuredMeshGenerator::generate_mesh( const StructuredGrid& rg, const std
                     }
                 }
 
+                if ( y_numbering > 0 ) {
+                    fix_quad_orientation( quad_nodes );
+                }
+
                 jcell = quad_begin + jquad++;
                 node_connectivity.set( jcell, quad_nodes );
                 cells_glb_idx( jcell ) = jcell + 1;
@@ -1185,6 +1261,9 @@ void StructuredMeshGenerator::generate_mesh( const StructuredGrid& rg, const std
                             triag_nodes[2] = node_numbering.at( offset_loc.at( ilatS ) );
                         }
                     }
+                    if ( y_numbering > 0 ) {
+                        fix_triag_orientation( triag_nodes );
+                    }
                 }
                 else  // This is a triangle pointing down
                 {
@@ -1202,6 +1281,9 @@ void StructuredMeshGenerator::generate_mesh( const StructuredGrid& rg, const std
                             triag_nodes[2] = node_numbering.at( offset_loc.at( ilatN ) );
                         }
                     }
+                    if ( y_numbering > 0 ) {
+                        fix_triag_orientation( triag_nodes );
+                    }
                 }
                 jcell = triag_begin + jtriag++;
                 node_connectivity.set( jcell, triag_nodes );
@@ -1224,61 +1306,103 @@ void StructuredMeshGenerator::generate_mesh( const StructuredGrid& rg, const std
             triag_nodes[0] = node_numbering.at( jnorth + ip1 );
             triag_nodes[1] = node_numbering.at( offset_loc.at( ilat ) + ip2 );
             triag_nodes[2] = node_numbering.at( offset_loc.at( ilat ) + ip3 );
+            if ( y_numbering > 0 ) {
+                fix_triag_orientation( triag_nodes );
+            }
             node_connectivity.set( jcell, triag_nodes );
             cells_glb_idx( jcell ) = jcell + 1;
             cells_part( jcell )    = mypart;
         }
     }
     else if ( patch_north_pole ) {
-        idx_t jlat = 0;
-        idx_t ilat = 0;
-        idx_t ip1, ip2, ip3;
+        idx_t jlat = y_numbering < 0 ? 0 : rg.ny() - 1;
+        idx_t ilat = y_numbering < 0 ? 0 : std::abs( region.south - region.north );
 
-        idx_t jforward  = 0;
-        idx_t jbackward = rg.nx( jlat ) - 1;
-        bool forward    = true;
+        if ( patch_quads ) {
+            idx_t ip1, ip2, ip3, ip4;
 
-        while ( true ) {
-            if ( forward ) {
+            idx_t jforward  = 0;
+            idx_t jbackward = rg.nx( jlat ) - 1;
+
+            while ( true ) {
                 ip1 = jforward;
                 ip2 = jforward + 1;
-                ip3 = jbackward;
-            }
-            else {
-                ip1 = jforward;
-                ip2 = jbackward - 1;
-                ip3 = jbackward;
-            }
+                ip3 = jbackward - 1;
+                ip4 = jbackward;
 
-            triag_nodes[0] = node_numbering.at( offset_loc.at( ilat ) + ip1 );
-            triag_nodes[1] = node_numbering.at( offset_loc.at( ilat ) + ip2 );
-            triag_nodes[2] = node_numbering.at( offset_loc.at( ilat ) + ip3 );
+                quad_nodes[0] = node_numbering.at( offset_loc.at( ilat ) + ip1 );
+                quad_nodes[1] = node_numbering.at( offset_loc.at( ilat ) + ip2 );
+                quad_nodes[2] = node_numbering.at( offset_loc.at( ilat ) + ip3 );
+                quad_nodes[3] = node_numbering.at( offset_loc.at( ilat ) + ip4 );
 
-            jcell = triag_begin + jtriag++;
-            node_connectivity.set( jcell, triag_nodes );
+                jcell = quad_begin + jquad++;
+                node_connectivity.set( jcell, quad_nodes );
 
-            cells_glb_idx( jcell ) = jcell + 1;
-            cells_part( jcell )    = mypart;
-            Topology::set( cells_flags( jcell ), Topology::PATCH );
+                cells_glb_idx( jcell ) = jcell + 1;
+                cells_part( jcell )    = mypart;
+                Topology::set( cells_flags( jcell ), Topology::PATCH );
 
-            if ( jbackward == jforward + 2 ) {
-                break;
-            }
-
-            if ( forward ) {
+                if ( jcell == cells_glb_idx.size() - 1 ) {
+                }
+                if ( jforward + 1 == jbackward ) {
+                    break;
+                }
                 ++jforward;
-                forward = false;
-            }
-            else {
                 --jbackward;
-                forward = true;
+            }
+        }
+        else {
+            idx_t ip1, ip2, ip3;
+
+            idx_t jforward  = 0;
+            idx_t jbackward = rg.nx( jlat ) - 1;
+            bool forward    = true;
+
+            while ( true ) {
+                if ( forward ) {
+                    ip1 = jforward;
+                    ip2 = jforward + 1;
+                    ip3 = jbackward;
+                }
+                else {
+                    ip1 = jforward;
+                    ip2 = jbackward - 1;
+                    ip3 = jbackward;
+                }
+
+                triag_nodes[0] = node_numbering.at( offset_loc.at( ilat ) + ip1 );
+                triag_nodes[1] = node_numbering.at( offset_loc.at( ilat ) + ip2 );
+                triag_nodes[2] = node_numbering.at( offset_loc.at( ilat ) + ip3 );
+                //            if ( y_numbering > 0 ) {
+                //                fix_triag_orientation( triag_nodes );
+                //            }
+
+                jcell = triag_begin + jtriag++;
+                node_connectivity.set( jcell, triag_nodes );
+
+                cells_glb_idx( jcell ) = jcell + 1;
+                cells_part( jcell )    = mypart;
+                Topology::set( cells_flags( jcell ), Topology::PATCH );
+
+                if ( jbackward == jforward + 2 ) {
+                    break;
+                }
+
+                if ( forward ) {
+                    ++jforward;
+                    forward = false;
+                }
+                else {
+                    --jbackward;
+                    forward = true;
+                }
             }
         }
     }
 
     if ( include_south_pole ) {
-        idx_t jlat = rg.ny() - 1;
-        idx_t ilat = region.south - region.north;
+        idx_t jlat = y_numbering > 0 ? 0 : rg.ny() - 1;
+        idx_t ilat = y_numbering > 0 ? 0 : std::abs( region.south - region.north );
         idx_t ip1  = 0;
         idx_t nlon = rg.nx( jlat ) + 1 - ( periodic_east_west ? 0 : 1 );
         for ( idx_t ip2 = 1; ip2 < nlon; ++ip2 ) {
@@ -1290,54 +1414,97 @@ void StructuredMeshGenerator::generate_mesh( const StructuredGrid& rg, const std
             if ( three_dimensional && ip2 == rg.nx( jlat ) ) {
                 triag_nodes[1] = node_numbering.at( offset_loc.at( ilat ) + 0 );
             }
+            if ( y_numbering > 0 ) {
+                fix_triag_orientation( triag_nodes );
+            }
+
             node_connectivity.set( jcell, triag_nodes );
             cells_glb_idx( jcell ) = jcell + 1;
             cells_part( jcell )    = mypart;
         }
     }
     else if ( patch_south_pole ) {
-        idx_t jlat = rg.ny() - 1;
-        idx_t ilat = region.south - region.north;
-        idx_t ip1, ip2, ip3;
+        idx_t jlat = y_numbering > 0 ? 0 : rg.ny() - 1;
+        idx_t ilat = y_numbering > 0 ? 0 : std::abs( region.south - region.north );
 
-        idx_t jforward  = 0;
-        idx_t jbackward = rg.nx( jlat ) - 1;
-        bool forward    = true;
+        if ( patch_quads ) {
+            idx_t ip1, ip2, ip3, ip4;
 
-        while ( true ) {
-            if ( forward ) {
-                ip1 = jforward;
-                ip2 = jforward + 1;
-                ip3 = jbackward;
-            }
-            else {
-                ip1 = jforward;
+            idx_t jforward  = 0;
+            idx_t jbackward = rg.nx( jlat ) - 1;
+
+            while ( true ) {
+                ip1 = jbackward;
                 ip2 = jbackward - 1;
-                ip3 = jbackward;
-            }
+                ip3 = jforward + 1;
+                ip4 = jforward;
 
-            triag_nodes[0] = node_numbering.at( offset_loc.at( ilat ) + ip1 );
-            triag_nodes[1] = node_numbering.at( offset_loc.at( ilat ) + ip2 );
-            triag_nodes[2] = node_numbering.at( offset_loc.at( ilat ) + ip3 );
+                quad_nodes[0] = node_numbering.at( offset_loc.at( ilat ) + ip1 );
+                quad_nodes[1] = node_numbering.at( offset_loc.at( ilat ) + ip2 );
+                quad_nodes[2] = node_numbering.at( offset_loc.at( ilat ) + ip3 );
+                quad_nodes[3] = node_numbering.at( offset_loc.at( ilat ) + ip4 );
 
-            jcell = triag_begin + jtriag++;
-            node_connectivity.set( jcell, triag_nodes );
+                jcell = quad_begin + jquad++;
+                node_connectivity.set( jcell, quad_nodes );
 
-            cells_glb_idx( jcell ) = jcell + 1;
-            cells_part( jcell )    = mypart;
-            Topology::set( cells_flags( jcell ), Topology::PATCH );
+                cells_glb_idx( jcell ) = jcell + 1;
+                cells_part( jcell )    = mypart;
+                Topology::set( cells_flags( jcell ), Topology::PATCH );
 
-            if ( jbackward == jforward + 2 ) {
-                break;
-            }
-
-            if ( forward ) {
+                if ( jcell == cells_glb_idx.size() - 1 ) {
+                }
+                if ( jforward + 1 == jbackward ) {
+                    break;
+                }
                 ++jforward;
-                forward = false;
-            }
-            else {
                 --jbackward;
-                forward = true;
+            }
+        }
+        else {
+            idx_t ip1, ip2, ip3;
+
+            idx_t jforward  = 0;
+            idx_t jbackward = rg.nx( jlat ) - 1;
+            bool forward    = true;
+
+            while ( true ) {
+                if ( forward ) {
+                    ip1 = jbackward;
+                    ip2 = jforward + 1;
+                    ip3 = jforward;
+                }
+                else {
+                    ip1 = jbackward;
+                    ip2 = jbackward - 1;
+                    ip3 = jforward;
+                }
+
+                triag_nodes[0] = node_numbering.at( offset_loc.at( ilat ) + ip1 );
+                triag_nodes[1] = node_numbering.at( offset_loc.at( ilat ) + ip2 );
+                triag_nodes[2] = node_numbering.at( offset_loc.at( ilat ) + ip3 );
+                //            if ( y_numbering > 0 ) {
+                //                fix_triag_orientation( triag_nodes );
+                //            }
+
+                jcell = triag_begin + jtriag++;
+                node_connectivity.set( jcell, triag_nodes );
+
+                cells_glb_idx( jcell ) = jcell + 1;
+                cells_part( jcell )    = mypart;
+                Topology::set( cells_flags( jcell ), Topology::PATCH );
+
+                if ( jbackward == jforward + 2 ) {
+                    break;
+                }
+
+                if ( forward ) {
+                    ++jforward;
+                    forward = false;
+                }
+                else {
+                    --jbackward;
+                    forward = true;
+                }
             }
         }
     }
