@@ -13,11 +13,13 @@
 /// @file NormaliseLongitude.h
 
 namespace atlas {
+namespace util {
 
 class NormaliseLongitude {
 public:
     // Normalise longitude between (west - eps, east - eps ) with west = 0., east = 360.
-    constexpr NormaliseLongitude() : west_( -eps_ ), east_( 360. - eps_ ) {}
+  // constexpr NormaliseLongitude() : west_( -eps_ ), east_( 360. - eps_) {}
+  constexpr NormaliseLongitude() : west_( -eps_ ), east_( 359.99999999999 ) {}
 
     // Normalise longitude between ( west-eps, east-eps )  with east = west + 360
     constexpr NormaliseLongitude( double west ) : west_( west - eps_ ), east_( west + 360. - eps_ ) {}
@@ -25,6 +27,8 @@ public:
     // Normalise longitude between ( west-eps, east+eps )
     constexpr NormaliseLongitude( double west, double east ) : west_( west - eps_ ), east_( east + eps_ ) {}
 
+    constexpr NormaliseLongitude( const NormaliseLongitude& other ) : west_( other.west_ ), east_( other.east_ ) {}
+
     double operator()( double lon ) const {
         while ( lon < west_ ) {
             lon += 360.;
@@ -36,11 +40,12 @@ public:
     }
 
 private:
-    const double west_;
-    const double east_;
+    double west_;
+    double east_;
 
 public:
     static constexpr double eps_ = 1.e-11;
 };
 
+}  // namespace util
 }  // namespace atlas
